
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <usb.h>
#include <time.h>

/* vendor and product id */
#define USB_VID  0x0403
#define USB_PID  0xc630

/* command format:
 * 7 6 5 4 3 2 1 0
 * D D D C C C L L
 *
 * DDD = Device
 * CCC = Command
 * LL  = number of bytes in transfer - 1
 */


/* Devices */
#define BLIT_BOARD			(0<<5)
#define LM75_BOARD			(2<<5)

/* BLIT_BOARD commands */
#define BLIT_BOARD_ECHO			(BLIT_BOARD | (0<<2))
#define BLIT_BOARD_GET_IDX		(BLIT_BOARD | (1<<2))

/* LM75_BOARD commands */
#define LM75_BOARD_READ			(LM75_BOARD | (0<<2))
#define LM75_BOARD_READ_ALL_FIRST	(LM75_BOARD | (1<<2))
#define LM75_BOARD_READ_ALL_NEXT	(LM75_BOARD | (2<<2))


/* commandline */
#define CMD_ECHO		"echo"
#define CMD_GET_IDX		"read_idx"
#define CMD_LM75_READ		"lm75_read"
#define CMD_LM75_READ_ALL	"lm75_read_all"


#define FROMBCD(x)      (((x) >> 4) * 10 + ((x) & 0xf))
#define TOBCD(x)        (((x) / 10 * 16) + ((x) % 10))

#define VAL_NUMB 288		// reicht für 24h jede 5min 2Byte


usb_dev_handle      *handle = NULL;


//****************************************
//* Hilfeausgabe
void prog_help(void)
{
	printf("Syntax:\n");
	printf("=======\n");
	printf("usb2blitboard echo\n");
	printf("usb2blitboard read_idx\n");
	printf("usb2blitboard lm75_read\n");
	printf("usb2blitboard lm75_read_all\n");
}


/* ---------------------------------------------------------------------- */
int myusb_send(int request, int value, int index) {
  if(usb_control_msg(handle, USB_TYPE_VENDOR, request, 
		      value, index, NULL, 0, 1000) < 0) {
    fprintf(stderr, "send->USB request failed!\n");
    return -1;
  }
  return 0;
}

/* to increase performance, a little buffer is being used to */
/* collect command bytes of the same type before transmitting them */
#define BUFFER_MAX_CMD 4        /* current protocol supports up to 4 bytes */
int buffer_current_type = -1;   /* nothing in buffer yet */
int buffer_current_fill = 0;    /* -"- */
unsigned char buffer[BUFFER_MAX_CMD];


/* ---------------------------------------------------------------------- */
/* flush command queue due to buffer overflow / content */
/* change or due to explicit request */
void myusb_flush(void) {
  int request, value, index;
  
  /* anything to flush? ignore request if not */
  if (buffer_current_type == -1)
    return;
  
  /* build request byte */
  request = buffer_current_type | (buffer_current_fill - 1);
  
  /* fill value and index with buffer contents. endianess should IMHO not */
  /* be a problem, since usb_control_msg() will handle this. */
  value = buffer[0] | (buffer[1] << 8);
  index = buffer[2] | (buffer[3] << 8);
  
  /* send current buffer contents */
  myusb_send(request, value, index);
  
  /* buffer is now free again */
  buffer_current_type = -1;
  buffer_current_fill = 0;
}

/* ---------------------------------------------------------------------- */
/* enqueue a command into the buffer */
void myusb_enqueue(int command_type, int value) {
  if ((buffer_current_type >= 0) && (buffer_current_type != command_type))
    myusb_flush();
  
  /* add new item to buffer */
  buffer_current_type = command_type;
  buffer[buffer_current_fill++] = value;
  
  /* flush buffer if it's full */
  if (buffer_current_fill == BUFFER_MAX_CMD)
    myusb_flush();
}


/* ---------------------------------------------------------------------- */
#define ECHO_NUM 100
void blit_board_echo(void) {

  int i, nBytes, errors=0;
  unsigned short val, ret;

  for(i=0;i<ECHO_NUM;i++) {
    val = rand() & 0xffff;
    
    nBytes = usb_control_msg(handle, 
	   USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_IN, 
	   BLIT_BOARD_ECHO, val, 0, (char*)&ret, sizeof(ret), 1000);

    if(nBytes < 0) {
      fprintf(stderr, "echo->USB request failed (%s)!\n", (char*)&ret);
      return;
    }

    if(val != ret)
      errors++;
  }

  if(errors) 
    fprintf(stderr, "ERROR: %d out of %d echo transfers failed!\n", 
	    errors, ECHO_NUM);
  else 
    printf("Echo test successful!\n");
}


/* ---------------------------------------------------------------------- */
/* get a value from the interface */
int myusb_get(unsigned char cmd) {
  unsigned char       buffer[2];
  int                 nBytes;

  /* send control request and accept return value */
  nBytes = usb_control_msg(handle, 
	   USB_TYPE_VENDOR | USB_RECIP_DEVICE | USB_ENDPOINT_IN, 
	   cmd, 0, 0, (char *)buffer, sizeof(buffer), 1000);

  if(nBytes < 0) {
    fprintf(stderr, "get->USB request failed!\n");
    return -1;
  }

  return buffer[0] + 256*buffer[1];
}


void print_temp(int val) {

	unsigned char h_byte;
	unsigned char l_byte;
	char * s;

	h_byte = val&0xff;
	l_byte = val>>8;

	if (h_byte >= 128) {			// wenn Bit7 gesetzt, dann negative Temp.
		h_byte = ~(h_byte);		// dann (lt. Datenblatt) Byte invertieren
		h_byte++;			// und plus 1
		s= "-";
	} else {
		s= "+";
	}
	if (l_byte >= 128) {			// wenn Bit7 gesetzt, dann ,5 sonst ,0
		l_byte=5;
	} else {
		l_byte=0;
	}

	printf("%s%d.%d°C\n", s, h_byte, l_byte);
}

/* ---------------------------------------------------------------------- */
void lm75_board_read(void) {

	print_temp(myusb_get(LM75_BOARD_READ));
}



/* ---------------------------------------------------------------------- */
void lm75_board_read_all(void) {

	int i;

	print_temp(myusb_get(LM75_BOARD_READ_ALL_FIRST));
	for(i=0;i<(VAL_NUMB-1);i++) {
		print_temp(myusb_get(LM75_BOARD_READ_ALL_NEXT));
	}
}

/* ---------------------------------------------------------------------- */
void blit_board_get_idx(void) {

	int val = myusb_get(BLIT_BOARD_GET_IDX);
	val = ((val&0xff) * 256) + (val>>8);
	printf("val_idx=%d\n", val);
}



/* ---------------------------------------------------------------------- */
/* ---------------------------------------------------------------------- */
int main(int argc, char *argv[]) {
	struct usb_bus      *bus;
	struct usb_device   *dev;

	printf("usb2blitboard; Uwe Berger, 2008\n");

	usb_init();
  	usb_find_busses();
	usb_find_devices();
  
	for(bus = usb_get_busses(); bus; bus = bus->next) {
		for(dev = bus->devices; dev; dev = dev->next) {
			if ((dev->descriptor.idVendor == USB_VID) && 
			    (dev->descriptor.idProduct == USB_PID)) {
				printf("Found BLIT-Board device on bus %s device %s.\n", 
					bus->dirname, dev->filename);
				/* open device */
				if(!(handle = usb_open(dev))) 
					fprintf(stderr, "Error: Cannot open BLIT-Board device: %s\n", 
						usb_strerror());
			break;
			}
		}
	}
  
	if(!handle) {
		fprintf(stderr, "Error: Could not find BLIT-Board device\n");
		exit(-1);
	}

	//* Befehlsverzweigung (1.Argument ist das Schluesselwort)
	if (argc > 1) 
	{
		if (strcmp(argv[1], CMD_LM75_READ) == 0) {
			lm75_board_read();
		}
		else if (strcmp(argv[1], CMD_LM75_READ_ALL) == 0) {
			lm75_board_read_all();
		}
		else if (strcmp(argv[1], CMD_ECHO) == 0) {
			blit_board_echo();
		}
		else if (strcmp(argv[1], CMD_GET_IDX) == 0) {
			blit_board_get_idx();

		} else {
			printf("usb2blitboard: unbekannter Befehl!\n");
			prog_help();
		}
	} else {
		printf("usb2blitboard: zu wenig Argumente!\n");
		prog_help();
	}

  usb_close(handle);

  return 0;
}
