
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include <avr/eeprom.h>

#include <util/delay.h>

#include "i2cmaster.h"
#include "i2clcd.h"

// use usbtiny library 
#include "usb.h"
#include "usbtiny.h"
typedef byte_t uchar;

#define USBDDR DDRD
#define USB_CFG_IOPORT PORTD

#define USB_CFG_DMINUS_BIT USBTINY_DMINUS
#define USB_CFG_DPLUS_BIT USBTINY_DPLUS

#define USBMASK (_BV(USBTINY_DPLUS) | _BV(USBTINY_DMINUS))

#define usbInit()  usb_init()
#define usbPoll()  usb_poll()


#define LED_DDR  DDRB		// Hardwarekonfiguration
#define LED_PORT PORTB
#define LED_PIN  PB1

#define PRELOAD  18661
#define TICKS    300
#define VAL_NUMB 288		// reicht für 24h jede 5min 2Byte


#define ADDR_LM75 	0x92

#define FROMBCD(x)      (((x) >> 4) * 10 + ((x) & 0xf))
#define TOBCD(x)        (((x) / 10 * 16) + ((x) % 10))

volatile unsigned char h_byte;	// Hight-Byte Temperatur-Registers LM75
volatile unsigned char l_byte;	// Low-Byte Temperatur-Registers LM75
volatile int click = TICKS-1;

volatile unsigned char values[VAL_NUMB*2];
volatile int val_idx = -2;
volatile int read_idx = 0;


/* -------------------------------------------------------------------------- */
void long_delay(uint16_t ms) {				// lange Pause
    for (; ms>0; ms--) _delay_ms(1);
}

/* -------------------------------------------------------------------------- */
ISR(TIMER1_OVF_vect)
{
	TCNT1     = PRELOAD;				// Preloadwert setzen
	LED_PORT ^= (1 << LED_PIN); 			// LED-PIN invertieren
	click++;
}

/* -------------------------------------------------------------------------- */
unsigned char lm75_init(void)
{
	if (i2c_start(ADDR_LM75+I2C_WRITE)) return 1;
	i2c_write(0);					// Funktion Temperatur lesen
	i2c_stop();
	long_delay(100);				// etwas warten bevor es weiter geht...
	return 0;
}

/* -------------------------------------------------------------------------- */
unsigned char lm75_read(void)
{
	if (i2c_start(ADDR_LM75+I2C_READ)) return 1;
	h_byte = i2c_readAck();
	l_byte = i2c_readNak();
	i2c_stop();
	return 0;
}

/* -------------------------------------------------------------------------- */
extern	byte_t	usb_setup ( byte_t data[8] )
{
	byte_t *replyBuf = data;
	uchar device = (data[1] >> 5);
	uchar command = (data[1] >> 2) & 3; // command 0 .. 7

	/* command format:
	 * 7 6 5 4 3 2 1 0
	 * D D D C C C L L
	 *
	 * DDD = Device
	 * CCC = Command
	 * LL  = number of bytes in transfer - 1
	 */

	switch (device) {

	case 0: // blit_board
		switch (command) {

		case 0: // echo-Test
			replyBuf[0] = data[2];
			replyBuf[1] = data[3];
			return 2;
			break;

		case 1: // Anzeige val_idx
			replyBuf[0] = (val_idx >> 8);
			replyBuf[1] = (val_idx & 0xff);
			return 2;
			break;

		default:
			break; 

		}
		break;

	case 2: // LM75_Board
    		switch(command) {

	  	case 0: // lm75_read
			lm75_read();
			replyBuf[0] = h_byte;
			replyBuf[1] = l_byte;
			return 2;
			break;

	  	case 1: // 1.Wert aus Messreihe
			read_idx = val_idx+2;
			if (read_idx >= VAL_NUMB*2) {
				read_idx = 0;
			}
			replyBuf[0] = values[read_idx];
			replyBuf[1] = values[read_idx+1];
			return 2;
			break;

	  	case 2: // Folgewert aus Messreihe
			read_idx = read_idx + 2;
			if (read_idx >= VAL_NUMB*2) {
				read_idx = 0;
			}
			replyBuf[0] = values[read_idx];
			replyBuf[1] = values[read_idx+1];
			return 2;
			break;

		default:
			break; 
     
		}
		break;

	default:
	    break;
	}

  return 0;  // reply len
}

/* ------------------------------------------------------------------------- */
/* ------------------------------------------------------------------------- */
int	main(void) {

	LED_DDR   = (1 << LED_PIN);		// LED-Pin als Ausgang schalten
	LED_PORT &= ~(1 << LED_PIN);

	TCNT1   = PRELOAD;			// Preloadwert setzen
	TCCR1B |= (1 << CS02);			// Prescaler Timer1 auf 256 stellen
	TIMSK  |= (1 << TOIE1);			// Timer 1 Overflow Interrupt enable


	usbInit();				// USB initialisieren
	i2c_init();				// TWI initialisieren
	lm75_init();				// Temperaturfuehler (LM75) initialisieren	

	sei();

	for(;;) {	
		usbPoll();
		
		if (click > TICKS) {
			click = 0;
			val_idx = val_idx + 2;
			if (val_idx >= VAL_NUMB*2) {
				val_idx = 0;
			}
			// LM75 auslesen/aufbereiten
			lm75_read();
			values[val_idx]   = h_byte;
			values[val_idx+1] = l_byte;
		}
	}
	return 0;
}

/* ------------------------------------------------------------------------- */
